package ru.ispras.kmb.spring8;

import java.io.IOException;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

public class RemoteTesterImpl implements RemoteTester, Subscriber {
	private Broker broker;
	private HashSet<String> topics = new HashSet<>();

	public void setBroker(InetAddress inetAddress, int port) throws RemoteException {
		try {
			broker = NetworkBroker.connect(inetAddress, port);
		} catch (IOException e) {
			e.printStackTrace();
			throw new RemoteException("Failed to connect to the brocker", e);
		}
	}
	public int subscribe(String topic) throws RemoteException {
		broker.subscribe(this, topic);
		topics.add(topic);
		return 1;
	}
	public int unsubscribe(String topic) throws RemoteException {
		broker.unsubscribe(this, topic);
		topics.remove(topic);
		return 1;
	}
	public String[] getTopics() throws RemoteException {
		return (String[])topics.toArray();
	}
	
	public int publish(String topic, String message) throws RemoteException {
		broker.publish(topic, message);
		return 1;
	}
	
	private List<ReceivedMessage> history = null;
	
	public List<ReceivedMessage> getReceivedMessages(boolean clear) 
			throws RemoteException {
		synchronized (this) {
			List<ReceivedMessage> result = history;
			history = null;
			return result;
		}
	}
	@Override
	public void receive(String topic, String msg) {
		synchronized (this) {
			if (history == null) 
				history = new ArrayList<ReceivedMessage>();
			history.add(new ReceivedMessage(topic, msg, 
					broker.getAddress()));
		}
	}
	
	public static void main(String args[]) {
        
        try {
            RemoteTesterImpl obj = new RemoteTesterImpl();
            RemoteTester stub = (RemoteTester) UnicastRemoteObject.exportObject(obj, 0);

            // Bind the remote object's stub in the registry
            Registry registry = LocateRegistry.createRegistry(54321);
            registry.bind("RemoteTester", stub);

            System.err.println("Tester ready");
        } catch (Exception e) {
            System.err.println("Tester exception: " + e.toString());
            e.printStackTrace();
            System.exit(1);
        }
    }
	
}
