package ru.ispras.kmb.spring8;

import static org.junit.Assert.*;

import java.net.Inet4Address;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.List;

import org.junit.Test;

public class TestRemote1 {

	private static final String HELLO_WORLD_MESSAGE = "hello, world!";
	private static final String TEST1_TOPIC = "/test1";

	@Test
	public void test() throws RemoteException, NotBoundException, UnknownHostException, InterruptedException {
		Registry registry = LocateRegistry.getRegistry(54321);
		RemoteTester tester = (RemoteTester) registry.lookup("RemoteTester");
		Broker b = NetworkBroker.connect(Inet4Address.getLocalHost(), 12345);
		
		tester.setBroker(Inet4Address.getLocalHost(), 12345);
		int res = tester.subscribe(TEST1_TOPIC);
		assertEquals(1, res);
		b.publish(TEST1_TOPIC, HELLO_WORLD_MESSAGE);
		
		Thread.sleep(500);
		List<ReceivedMessage> msgs = tester.getReceivedMessages(true);
		assertNotNull(msgs);
		assertTrue("No messages received", 0 < msgs.size());
		
		int found = 0;
		for (ReceivedMessage msg : msgs) {
			if (!TEST1_TOPIC.equals(msg.topic)) continue;
			if (!HELLO_WORLD_MESSAGE.equals(msg.message)) continue;
			if (!Inet4Address.getLocalHost().equals(msg.broker.getAddress())) continue;
			if (12345 != msg.broker.getPort()) continue;
			found ++;
		}
		assertEquals(1, found);
	}

}
